/*
 * fslurp, Copyright (c) 2009 David Stone <ds.fslurp@asif.com>
 *
 * $RCSfile: display.c,v $
 * $Revision: 1.5 $
 * $Date: 2010/05/26 17:55:34 $
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation.  No representations are made about the suitability of this
 * software for any purpose.  It is provided "as is" without express or 
 * implied warranty.
 */

#include "fslurp.h"

const char* getTimestamp()
{
	static char	ctimeBuf[4096];
	struct tm	*tmResult;

	tmResult = localtime(&InverterGlobals.timestamp);
	strftime(ctimeBuf, sizeof(ctimeBuf), TimeFormat, tmResult);

	return ctimeBuf;
}

// FIX THIS, DS: clean up and condense
const char* getValue(struct MeasuredValue measuredValue)
{
	unsigned int	value;
	static char	valueString[64];
	signed char	exponent;

	exponent = measuredValue.exponent;

	if (exponent == 0xb)
	{
		return ("overflow");
	}
	else if (exponent == -4)
	{
		return ("underflow");
	}

	valueString[0] = 0;

	value = measuredValue.lsb;
	value += ((measuredValue.msb) << 8);

	if (exponent == 0)
	{
		sprintf(valueString, "%d", value);
		return valueString;
	}
	else if (exponent > 0)
	{
		sprintf(valueString, "%d", value * (int)(pow(10, exponent)));
	}
	else
	{
		assert(exponent < 0);

		double	gValue;

		gValue = value;
		gValue *= pow(10, exponent);

		sprintf(valueString, "%.2f", gValue);
	}

	return valueString;
}

const char* getThousandsValue(struct MeasuredValue measuredValue)
{
	unsigned int	value;
	static char	valueString[64];
	signed char	exponent;

	exponent = measuredValue.exponent;

	if (exponent == 0xb)
	{
		return ("overflow");
	}
	else if (exponent == -4)
	{
		return ("underflow");
	}

	valueString[0] = 0;

	value = measuredValue.lsb;
	value += ((measuredValue.msb) << 8);

	exponent -= 3;

	if (exponent == 0)
	{
		sprintf(valueString, "%d", value);
		return valueString;
	}
	else if (exponent > 0)
	{
		sprintf(valueString, "%d", value * (int)(pow(10, exponent)));
	}
	else
	{
		assert(exponent < 0);

		double	gValue;

		gValue = value;
		gValue *= pow(10, exponent);

		sprintf(valueString, "%.2f", gValue);
	}

	return valueString;
}

const char* getHoursValue(struct MeasuredValue measuredValue)
{
	double		gValue;
	static char	valueString[64];
	signed char	exponent;

	exponent = measuredValue.exponent;

	if (exponent == 0xb)
	{
		return ("overflow");
	}
	else if (exponent == -4)
	{
		return ("underflow");
	}

	valueString[0] = 0;

	gValue = measuredValue.lsb;
	gValue += ((measuredValue.msb) << 8);
	gValue /= 60;
	gValue *= pow(10, exponent);
	sprintf(valueString, "%.2f", gValue);

	return valueString;
}

void displayTimestampHeader()
{
	printf("Time%s", Delimiter);
}

void displayTimestamp()
{
	if (Delimiter == NULL)
	{
		printf("Time			: %s\n", getTimestamp());
	}
	else
	{
		printf("%s%s", getTimestamp(), Delimiter);
	}
}

void displayInverterGlobalsHeader(void)
{
	printf("Time%s", Delimiter);
	printf("Interface card type%s", Delimiter);
	printf("Interface card version%s", Delimiter);
	printf("Active inverter number%s", Delimiter);
	printf("Inverter type%s", Delimiter);
}

void displayInverterGlobals(void)
{
	if (Delimiter == NULL)
	{
		printf("Interface card type	: %#02x\n",
			InverterGlobals.ifcType);

		printf("Interface card version	: %d.%d.%d\n",
			InverterGlobals.versionMajor,
			InverterGlobals.versionMinor,
			InverterGlobals.versionRelease);

		printf("Active inverter number	: %#0x\n",
			InverterGlobals.activeInverter);

		printf("Inverter type		: %s\n",
			getInverterModel(InverterGlobals.deviceType));
	}
	else
	{
		printf("%d%s", InverterGlobals.ifcType, Delimiter);

		printf("%d.%d.%d%s",
			InverterGlobals.versionMajor,
			InverterGlobals.versionMinor,
			InverterGlobals.versionRelease,
			Delimiter);

		printf("%#02x%s", InverterGlobals.activeInverter, Delimiter);

		printf("%s%s",
			getInverterModel(InverterGlobals.deviceType),
			Delimiter);
	}
}

void displayInverterNowHeader()
{
	printf("Power now%s", Delimiter);
	printf("AC current now%s", Delimiter);
	printf("AC voltage now%s", Delimiter);
	printf("AC frequency now%s", Delimiter);
	printf("DC current now%s", Delimiter);
	printf("DC voltage now");

	if (ReportType == ReportTypeAll)
	{
		printf("%s", Delimiter);
	}
	else
	{
		assert(ReportType == ReportTypeNow);
		printf("\n");
	}
}

void displayInverterNowData(struct InverterNowData data)
{
	if (Delimiter == NULL)
	{
		printf("Power now		: %s watts\n",
			getValue(data.power));
		printf("AC current now		: %s amps\n",
			getValue(data.acCurrent));
		printf("AC voltage now		: %s volts\n",
			getValue(data.acVoltage));
		printf("AC frequency now	: %s Hz\n",
			getValue(data.acFrequency));
		printf("DC current now		: %s amps\n",
			getValue(data.dcCurrent));
		printf("DC voltage now		: %s volts\n",
			getValue(data.dcVoltage));
	}
	else
	{
		printf("%s%s", getValue(data.power), Delimiter);
		printf("%s%s", getValue(data.acCurrent), Delimiter);
		printf("%s%s", getValue(data.acVoltage), Delimiter);
		printf("%s%s", getValue(data.acFrequency), Delimiter);
		printf("%s%s", getValue(data.dcCurrent), Delimiter);
		printf("%s", getValue(data.dcVoltage));

		if (ReportType == ReportTypeAll)
		{
			printf("%s", Delimiter);
		}
		else
		{
			assert(ReportType == ReportTypeNow);
			printf("\n");
		}
	}
}

void displayInverterDayHeader()
{
	printf("Energy today (KWH)%s", Delimiter);
	printf("Maximum power today (W)%s", Delimiter);
	printf("Maximum AC voltage today (V)%s", Delimiter);
	printf("Minimum AC voltage today (V)%s", Delimiter);
	printf("Maximum DC voltage today (V)%s", Delimiter);
	printf("Operating time today (H)");

	if (ReportType == ReportTypeAll)
	{
		printf("%s", Delimiter);
	}
	else
	{
		assert(ReportType == ReportTypeDay);
		printf("\n");
	}
}

void displayInverterDayData(struct InverterDayData data)
{
	if (Delimiter == NULL)
	{
		printf("Energy today		: %s kilowatt hours\n",
			getThousandsValue(data.energy));
		printf("Maximum power today	: %s watts\n",
			getValue(data.maximumPower));
		printf("Maximum AC voltage today: %s volts\n",
			getValue(data.acVoltageMaximum));
		printf("Minimum AC voltage today: %s volts\n",
			getValue(data.acVoltageMinimum));
		printf("Maximum DC voltage today: %s volts\n",
			getValue(data.dcVoltageMaximum));
		printf("Operating time today	: %s hours\n",
			getHoursValue(data.operatingHours));
	}
	else
	{
		printf("%s%s", getThousandsValue(data.energy), Delimiter);
		printf("%s%s", getValue(data.maximumPower), Delimiter);
		printf("%s%s", getValue(data.acVoltageMaximum), Delimiter);
		printf("%s%s", getValue(data.acVoltageMinimum), Delimiter);
		printf("%s%s", getValue(data.dcVoltageMaximum), Delimiter);
		printf("%s", getHoursValue(data.operatingHours));

		if (ReportType == ReportTypeAll)
		{
			printf("%s", Delimiter);
		}
		else
		{
			assert(ReportType == ReportTypeDay);
			printf("\n");
		}
	}
}

void displayInverterTotalHeader()
{
	printf("Energy total (KWH)%s", Delimiter);
	printf("Maximum power total (W)%s", Delimiter);
	printf("Maximum AC voltage total (V)%s", Delimiter);
	printf("Minimum AC voltage total (V)%s", Delimiter);
	printf("Maximum DC voltage total (V)%s", Delimiter);
	printf("Operating time total (H)\n");
}

void displayInverterTotalData(struct InverterTotalData data)
{
	if (Delimiter == NULL)
	{
		printf("Energy total		: %s kilowatt hours\n",
			getThousandsValue(data.energy));
		printf("Maximum power total	: %s watts\n",
			getValue(data.maximumPower));
		printf("Maximum AC voltage total: %s volts\n",
			getValue(data.acVoltageMaximum));
		printf("Minimum AC voltage total: %s volts\n",
			getValue(data.acVoltageMinimum));
		printf("Maximum DC voltage total: %s volts\n",
			getValue(data.dcVoltageMaximum));
		printf("Operating time total	: %s hours\n",
			getHoursValue(data.operatingHours));
	}
	else
	{
		printf("%s%s", getThousandsValue(data.energy), Delimiter);
		printf("%s%s", getValue(data.maximumPower), Delimiter);
		printf("%s%s", getValue(data.acVoltageMaximum), Delimiter);
		printf("%s%s", getValue(data.acVoltageMinimum), Delimiter);
		printf("%s%s", getValue(data.dcVoltageMaximum), Delimiter);
		printf("%s\n", getHoursValue(data.operatingHours));
	}
}

void inverterNowReport(struct InverterNowData data)
{
	displayTimestamp();
	displayInverterNowData(data);
}

void inverterDayReport(struct InverterDayData data)
{
	displayTimestamp();
	displayInverterDayData(data);
}

void inverterTotalReport(struct InverterTotalData data)
{
	displayTimestamp();
	displayInverterTotalData(data);
}

void inverterAllReport(
	struct InverterNowData nowData,
	struct InverterDayData dayData,
	struct InverterTotalData totalData)
{
	displayTimestamp();
	displayInverterGlobals();
	displayInverterNowData(nowData);
	displayInverterDayData(dayData);
	displayInverterTotalData(totalData);
}

void inverterHeaderReport(void)
{
	switch (ReportType)
	{
		case ReportTypeNow:
			displayTimestampHeader();
			displayInverterNowHeader();
			break;
		case ReportTypeDay:
			displayTimestampHeader();
			displayInverterDayHeader();
			break;
		case ReportTypeTotal:
			displayTimestampHeader();
			displayInverterTotalHeader();
			break;
		case ReportTypeAll:
			displayTimestampHeader();
			displayInverterGlobalsHeader();
			displayInverterNowHeader();
			displayInverterDayHeader();
			displayInverterTotalHeader();
			break;
		default:
			assert(0);
	}
}
