/*
 * fslurp, Copyright (c) 2009 David Stone <ds.fslurp@asif.com>
 *
 * $RCSfile: fetch.c,v $
 * $Revision: 1.3 $
 * $Date: 2012/09/02 19:06:18 $
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation.  No representations are made about the suitability of this
 * software for any purpose.  It is provided "as is" without express or 
 * implied warranty.
 */

#include "fslurp.h"

void getVersion(void)
{
	struct Request		request;
	struct VersionResponse*	response;

	request.startByte0 = FRONIUS_START_VALUE;
	request.startByte1 = FRONIUS_START_VALUE;
	request.startByte2 = FRONIUS_START_VALUE;
	request.length = 0;
	request.deviceOption = FRONIUS_INTERFACE_CARD;
	request.deviceNumber = 0;
	request.command = FRONIUS_CMD_GET_VERSION;
	request.checksum = calculateChecksum(
		&request.length,
		&request.checksum - &request.length);

	// Do an initial write and flush to give the Fronius
	// serial port a chance to auto detect the baud rate.

	writeBytes(&request);
	sleep(1);
	flushSerialPort();

	writeBytes(&request);
	readResponse();

	response = (struct VersionResponse*)ResponseBuffer;
	InverterGlobals.ifcType = response->ifcType;
	InverterGlobals.versionMajor = response->versionMajor;
	InverterGlobals.versionMinor = response->versionMinor;
	InverterGlobals.versionRelease = response->versionRelease;
}

void getDeviceType(void)
{
	struct Request			request;
	struct DeviceTypeResponse*	response;

	request.startByte0 = FRONIUS_START_VALUE;
	request.startByte1 = FRONIUS_START_VALUE;
	request.startByte2 = FRONIUS_START_VALUE;
	request.length = 0;
	request.deviceOption = FRONIUS_INVERTER;
	request.deviceNumber = InverterGlobals.activeInverter;
	request.command = FRONIUS_CMD_GET_DEVICE_TYPE;
	request.checksum = calculateChecksum(
		&request.length,
		&request.checksum - &request.length);

	writeBytes(&request);
	readResponse();

	response = (struct DeviceTypeResponse*)ResponseBuffer;
	assert(response->length == FRONIUS_DEVICE_TYPE_RESPONSE_LENGTH);

	InverterGlobals.deviceType = response->typeIdentification;
}

void getActiveInverter(void)
{
	struct Request			request;
	struct ActiveInverterResponse*	response;

	request.startByte0 = FRONIUS_START_VALUE;
	request.startByte1 = FRONIUS_START_VALUE;
	request.startByte2 = FRONIUS_START_VALUE;
	request.length = 0;
	request.deviceOption = FRONIUS_INTERFACE_CARD;
	request.deviceNumber = 0;
	request.command = FRONIUS_CMD_GET_ACTIVE_INVERTER_NUMBER;
	request.checksum = calculateChecksum(
		&request.length,
		&request.checksum - &request.length);

	writeBytes(&request);
	readResponse();

	response = (struct ActiveInverterResponse*)ResponseBuffer;

	if (response->length == 0)
	{
		dieMessage(EXIT_NO_INVERTER, "No active inverter found.\n");
		return;
	}

	assert(response->length == FRONIUS_ACTIVE_INVERTER_RESPONSE_LENGTH);
	InverterGlobals.activeInverter = response->activeInverter;
}


void getMeasuredValue(	
	unsigned char command,
	struct MeasuredValue* measuredValue)
{
	struct Request			request;
	struct ValueQueryResponse*	response;

	request.startByte0 = FRONIUS_START_VALUE;
	request.startByte1 = FRONIUS_START_VALUE;
	request.startByte2 = FRONIUS_START_VALUE;
	request.length = 0;
	request.deviceOption = FRONIUS_INVERTER;
	request.deviceNumber = InverterGlobals.activeInverter;
	request.command = command;
	request.checksum = calculateChecksum(
		&request.length,
		&request.checksum - &request.length);

	writeBytes(&request);
	readResponse();

	response = (struct ValueQueryResponse*)ResponseBuffer;
	assert(response->length == FRONIUS_VALUE_QUERY_RESPONSE_LENGTH);

	assert(measuredValue != NULL);
	measuredValue->lsb = response->lsb;
	measuredValue->msb = response->msb;
	measuredValue->exponent = response->exponent;
}

void getInverterNowData(struct InverterNowData* inverterData)
{
	getMeasuredValue(
		FRONIUS_CMD_GET_POWER_NOW,
		&inverterData->power);
	getMeasuredValue(
		FRONIUS_CMD_GET_AC_CURRENT_NOW,
		&inverterData->acCurrent);
	getMeasuredValue(
		FRONIUS_CMD_GET_AC_VOLTAGE_NOW,
		&inverterData->acVoltage);
	getMeasuredValue(
		FRONIUS_CMD_GET_AC_FREQUENCY_NOW,
		&inverterData->acFrequency);
	getMeasuredValue(
		FRONIUS_CMD_GET_DC_CURRENT_NOW,
		&inverterData->dcCurrent);
	getMeasuredValue(
		FRONIUS_CMD_GET_DC_VOLTAGE_NOW,
		&inverterData->dcVoltage);
}

// FIX THIS, DS: support day yield?
void getInverterDayData(struct InverterDayData* inverterData)
{
	getMeasuredValue(
		FRONIUS_CMD_GET_ENERGY_DAY,
		&inverterData->energy);
	getMeasuredValue(
		FRONIUS_CMD_GET_MAXIMUM_POWER_DAY,
		&inverterData->maximumPower);
	getMeasuredValue(
		FRONIUS_CMD_GET_MAXIMUM_AC_VOLTAGE_DAY,
		&inverterData->acVoltageMaximum);
	getMeasuredValue(
		FRONIUS_CMD_GET_MINIMUM_AC_VOLTAGE_DAY,
		&inverterData->acVoltageMinimum);
	getMeasuredValue(
		FRONIUS_CMD_GET_MAXIMUM_DC_VOLTAGE_DAY,
		&inverterData->dcVoltageMaximum);
	getMeasuredValue(
		FRONIUS_CMD_GET_OPERATING_HOURS_DAY,
		&inverterData->operatingHours);
}

// FIX THIS, DS: support total yield?
void getInverterTotalData(struct InverterTotalData* inverterData)
{
	getMeasuredValue(
		FRONIUS_CMD_GET_ENERGY_TOTAL,
		&inverterData->energy);
	getMeasuredValue(
		FRONIUS_CMD_GET_MAXIMUM_POWER_TOTAL,
		&inverterData->maximumPower);
	getMeasuredValue(
		FRONIUS_CMD_GET_MAXIMUM_AC_VOLTAGE_TOTAL,
		&inverterData->acVoltageMaximum);
	getMeasuredValue(
		FRONIUS_CMD_GET_MINIMUM_AC_VOLTAGE_TOTAL,
		&inverterData->acVoltageMinimum);
	getMeasuredValue(
		FRONIUS_CMD_GET_MAXIMUM_DC_VOLTAGE_TOTAL,
		&inverterData->dcVoltageMaximum);
	getMeasuredValue(
		FRONIUS_CMD_GET_OPERATING_HOURS_TOTAL,
		&inverterData->operatingHours);
}
