/*
 * fslurp, Copyright (c) 2009 David Stone <ds.fslurp@asif.com>
 *
 * $RCSfile: fronius.h,v $
 * $Revision: 1.1 $
 * $Date: 2009/10/24 23:54:33 $
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation.  No representations are made about the suitability of this
 * software for any purpose.  It is provided "as is" without express or 
 * implied warranty.
 */

#ifndef _FRONIUS_H_
#define _FRONIUS_H_

// Commands
#define FRONIUS_CMD_GET_VERSION				0x01
#define FRONIUS_CMD_GET_DEVICE_TYPE			0x02
#define FRONIUS_CMD_GET_DATE_TIME			0x03
#define FRONIUS_CMD_GET_ACTIVE_INVERTER_NUMBER		0x04
#define FRONIUS_CMD_GET_POWER_NOW			0x10
#define FRONIUS_CMD_GET_ENERGY_TOTAL			0x11
#define FRONIUS_CMD_GET_ENERGY_DAY			0x12
// FIX THIS, DS: supported?
#define FRONIUS_CMD_GET_ENERGY_YEAR			0x13
#define FRONIUS_CMD_GET_AC_CURRENT_NOW			0x14
#define FRONIUS_CMD_GET_AC_VOLTAGE_NOW			0x15
#define FRONIUS_CMD_GET_AC_FREQUENCY_NOW		0x16
#define FRONIUS_CMD_GET_DC_CURRENT_NOW			0x17
#define FRONIUS_CMD_GET_DC_VOLTAGE_NOW			0x18
// FIX THIS, DS: supported?
#define FRONIUS_CMD_GET_YIELD_DAY			0x19
#define FRONIUS_CMD_GET_MAXIMUM_POWER_DAY		0x1a
#define FRONIUS_CMD_GET_MAXIMUM_AC_VOLTAGE_DAY		0x1b
#define FRONIUS_CMD_GET_MINIMUM_AC_VOLTAGE_DAY		0x1c
#define FRONIUS_CMD_GET_MAXIMUM_DC_VOLTAGE_DAY		0x1d
#define FRONIUS_CMD_GET_OPERATING_HOURS_DAY		0x1e
// FIX THIS, DS: supported?
#define FRONIUS_CMD_GET_YIELD_YEAR			0x20
#define FRONIUS_CMD_GET_MAXIMUM_POWER_TOTAL		0x26
#define FRONIUS_CMD_GET_MAXIMUM_AC_VOLTAGE_TOTAL	0x27
#define FRONIUS_CMD_GET_MINIMUM_AC_VOLTAGE_TOTAL	0x28
#define FRONIUS_CMD_GET_MAXIMUM_DC_VOLTAGE_TOTAL	0x29
#define FRONIUS_CMD_GET_OPERATING_HOURS_TOTAL		0x2a

#define FRONIUS_ERROR					0x0e

// Protocol Error Info

#define FRONIUS_PROTOCOL_ERROR_UNKNOWN_COMMAND		0x01
#define FRONIUS_PROTOCOL_ERROR_TIMEOUT			0x02
#define FRONIUS_PROTOCOL_ERROR_INCORRECT_DATA_STRUCTURE	0x03
#define FRONIUS_PROTOCOL_ERROR_COMMAND_QUEUE_FULL	0x04
#define FRONIUS_PROTOCOL_ERROR_DEVICE_OR_OPTION_NA	0x05
#define FRONIUS_PROTOCOL_ERROR_NO_RESPONSE		0x06
#define FRONIUS_PROTOCOL_ERROR_SENSOR_ERROR		0x07
#define FRONIUS_PROTOCOL_ERROR_SENSOR_NOT_ACTIVE	0x08
#define FRONIUS_PROTOCOL_ERROR_INCORRECT_COMMAND	0x09
#define FRONIUS_PROTOCOL_ERROR_DEVICE_ID_COLLISION	0x0a

#define FRONIUS_START_OFFSET			0
#define FRONIUS_LENGTH_OFFSET			3		
#define FRONIUS_DEVICE_OPTION_OFFSET		4
#define FRONIUS_DEVICE_NUMBER_OFFSET		5
#define FRONIUS_COMMAND_OFFSET			6

#define FRONIUS_HEADER_LENGTH			7
#define FRONIUS_NUM_CHECKSUMMED_HDR_BYTES	4
#define FRONIUS_CHECKSUM_LENGTH			1

// values for the device/option byte
#define FRONIUS_INTERFACE_CARD			0
#define FRONIUS_INVERTER			1

#define FRONIUS_START_LENGTH			3
#define FRONIUS_START_VALUE			((unsigned char)0x80)

#define FRONIUS_VERSION_RESPONSE_LENGTH		4
#define FRONIUS_DEVICE_TYPE_RESPONSE_LENGTH	1
#define FRONIUS_ACTIVE_INVERTER_RESPONSE_LENGTH	1
#define FRONIUS_VALUE_QUERY_RESPONSE_LENGTH	3

#define DEFAULT_SERIAL_DEVICE			"/dev/ttyS0"
#define DEFAULT_BAUD_RATE			"9600"

#endif /* _FRONIUS_H_ */

