/*
 * fslurp, Copyright (c) 2009 David Stone <ds.fslurp@asif.com>
 *
 * $RCSfile: fslurp.h,v $
 * $Revision: 1.3 $
 * $Date: 2010/05/17 01:08:13 $
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation.  No representations are made about the suitability of this
 * software for any purpose.  It is provided "as is" without express or 
 * implied warranty.
 */

#ifndef _FSLURP_H
#define _FSLURP_H

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termios.h>
#include <assert.h>
#include <stdarg.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include <errno.h>

#include "fronius.h"

#define DEFAULT_SERIAL_DEVICE	"/dev/ttyS0"
#define DEFAULT_BAUD_RATE	"9600"
#define DEFAULT_REPORT_TYPE	"now"

#define EXIT_NORMAL		0
#define EXIT_COMM_FAILURE	1
#define EXIT_NO_INVERTER	2
#define EXIT_USAGE_ERROR	3
#define EXIT_PROTOCOL_ERROR	4

enum ReportTypeEnum
{
	ReportTypeNow,
	ReportTypeDay,
	ReportTypeTotal,
	ReportTypeAll
};

struct Request
{
	unsigned char	startByte0;
	unsigned char	startByte1;
	unsigned char	startByte2;
	unsigned char	length;
	unsigned char	deviceOption;
	unsigned char	deviceNumber;
	unsigned char	command;
	unsigned char	checksum;
};

struct ErrorResponse
{
	unsigned char	startByte0;
	unsigned char	startByte1;
	unsigned char	startByte2;
	unsigned char	length;
	unsigned char	deviceOption;
	unsigned char	deviceNumber;
	unsigned char	error;
	unsigned char	failedCommand;
	unsigned char	errorInformation;
	unsigned char	checksum;
};

struct VersionResponse
{
	unsigned char	startByte0;
	unsigned char	startByte1;
	unsigned char	startByte2;
	unsigned char	length;
	unsigned char	deviceOption;
	unsigned char	deviceNumber;
	unsigned char	command;
	unsigned char	ifcType;
	unsigned char	versionMajor;
	unsigned char	versionMinor;
	unsigned char	versionRelease;
	unsigned char	checksum;
};

struct DeviceTypeResponse
{
	unsigned char	startByte0;
	unsigned char	startByte1;
	unsigned char	startByte2;
	unsigned char	length;
	unsigned char	deviceOption;
	unsigned char	deviceNumber;
	unsigned char	command;
	unsigned char	typeIdentification;
	unsigned char	checksum;
};

struct ActiveInverterResponse
{
	unsigned char	startByte0;
	unsigned char	startByte1;
	unsigned char	startByte2;
	unsigned char	length;
	unsigned char	deviceOption;
	unsigned char	deviceNumber;
	unsigned char	command;
	unsigned char	activeInverter;
	unsigned char	checksum;
};

struct ValueQueryResponse
{
	unsigned char	startByte0;
	unsigned char	startByte1;
	unsigned char	startByte2;
	unsigned char	length;
	unsigned char	deviceOption;
	unsigned char	deviceNumber;
	unsigned char	command;
	unsigned char	msb;
	unsigned char	lsb;
	unsigned char	exponent;
	unsigned char	checksum;
};

struct GlobalData
{
	time_t		timestamp;
	unsigned char	ifcType;
	unsigned char	versionMajor;
	unsigned char	versionMinor;
	unsigned char	versionRelease;
	unsigned char	activeInverter;
	unsigned char	deviceType;
};

struct MeasuredValue
{
	unsigned char	lsb;
	unsigned char	msb;
	signed char	exponent;
};

struct InverterNowData
{
	struct MeasuredValue	power;
	struct MeasuredValue	acCurrent;
	struct MeasuredValue	acVoltage;
	struct MeasuredValue	acFrequency;
	struct MeasuredValue	dcCurrent;
	struct MeasuredValue	dcVoltage;
};

struct InverterDayData
{
	struct MeasuredValue	energy;
	struct MeasuredValue	maximumPower;
	struct MeasuredValue	acVoltageMaximum;
	struct MeasuredValue	acVoltageMinimum;
	struct MeasuredValue	dcVoltageMaximum;
	struct MeasuredValue	operatingHours;
};

struct InverterTotalData
{
	struct MeasuredValue	energy;
	struct MeasuredValue	maximumPower;
	struct MeasuredValue	acVoltageMaximum;
	struct MeasuredValue	acVoltageMinimum;
	struct MeasuredValue	dcVoltageMaximum;
	struct MeasuredValue	operatingHours;
};

#include "externs.h"

#endif /* _FSLURP_H */

