/*
 * fslurp, Copyright (c) 2009 David Stone <ds.fslurp@asif.com>
 *
 * $RCSfile: inverters.c,v $
 * $Revision: 1.3 $
 * $Date: 2012/09/02 19:07:54 $
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation.  No representations are made about the suitability of this
 * software for any purpose.  It is provided "as is" without express or 
 * implied warranty.
 */

#include "fslurp.h"

#define NUM_ELEMENTS(x)	(sizeof((x)) / sizeof(*(x)))

struct Inverter
{
	unsigned char	id;
	char*		model;
};

struct Inverter Inverters[] =
{
	{ 0xFE, "FRONIUS IG 15 1-phase inverter" },
	{ 0xFD, "FRONIUS IG 20 1-phase inverter" },
	{ 0xFC, "FRONIUS IG 30 1-phase inverter" },
	{ 0xFB, "FRONIUS IG 30 Dummy, Dummy inverter" },
	{ 0xFA, "FRONIUS IG 40 1-phase inverter" },
	{ 0x88, "FRONIUS IG 50 1-phase inverter" },
	{ 0xF9, "FRONIUS IG 60 / IG 60 HV, 1-phase inverter" },
	{ 0xF6, "FRONIUS IG 300 3-phase inverter" },
	{ 0xF5, "FRONIUS IG 400 3-phase inverter" },
	{ 0xF4, "FRONIUS IG 500 3-phase inverter" },
	{ 0xF3, "FRONIUS IG 60 / IG 60 HV, 1-phase inverter" },
	{ 0xEE, "FRONIUS IG 2000 1-phase inverter" },
	{ 0xED, "FRONIUS IG 3000 1-phase inverter" },
	{ 0xEB, "FRONIUS IG 4000 1-phase inverter" },
	{ 0xEA, "FRONIUS IG 5100 1-phase inverter" },
	{ 0xE5, "FRONIUS IG 2500-LV 1-phase inverter" },
	{ 0xE3, "FRONIUS IG 4500-LV 1-phase inverter" },
	{ 0xDF, "Fronius IG Plus 11.4-3 Delta 3-phase inverter" },
	{ 0xDE, "Fronius IG Plus 11.4-1 UNI 1-phase inverter" },
	{ 0xDD, "Fronius IG Plus 10.0-1 UNI 1-phase inverter" },
	{ 0xDC, "Fronius IG Plus 7.5-1 UNI 1-phase inverter" },
	{ 0xDB, "Fronius IG Plus 6.0-1 UNI 1-phase inverter" },
	{ 0xDA, "Fronius IG Plus 5.0-1 UNI 1-phase inverter" },
	{ 0xD9, "Fronius IG Plus 3.8-1 UNI 1-phase inverter" },
	{ 0xD8, "Fronius IG Plus 3.0-1 UNI 1-phase inverter" },
	{ 0xD7, "Fronius IG Plus 120-3 3-phase inverter" },
	{ 0xD6, "Fronius IG Plus 70-2 2-phase inverter" },
	{ 0xD5, "Fronius IG Plus 70-1 1-phase inverter" },
	{ 0xD4, "Fronius IG Plus 35-1 1-phase inverter" },
	{ 0xD3, "Fronius IG Plus 150-3 3-phase inverter" },
	{ 0xD2, "Fronius IG Plus 100-2 2-phase inverter" },
	{ 0xD1, "Fronius IG Plus 100-1 1-phase inverter" },
	{ 0xD0, "Fronius IG Plus 50-1 1-phase inverter" },
	{ 0xCF, "Fronius IG Plus 12.0-3 WYE277 3-phase inverter" },
	{ 0xC1, "Fronius IG TL 3.6 1-phase inverter" },
	{ 0xC0, "Fronius IG TL 5.0 1-phase inverter" },
	{ 0xBF, "Fronius IG TL 4.0 1-phase inverter" },
	{ 0xBE, "Fronius IG TL 3.0 1-phase inverter" },
	{ 0x3C, "Fronius IG TL 4.6 1-phase inverter" },
	{ 0x2C, "Fronius IG TL Dummy, Dummy inverter" },
	{ 0x83, "Fronius IG Plus 25V-1 1-phase inverter" },
	{ 0x89, "Fronius IG Plus 30V-1 1-phase inverter" },
	{ 0xB1, "Fronius IG Plus 35V-1 1-phase inverter" },
	{ 0xB0, "Fronius IG Plus 50V-1 1-phase inverter" },
	{ 0xAF, "Fronius IG Plus 70V-1 1-phase inverter" },
	{ 0xAE, "Fronius IG Plus 70V-2 2-phase inverter" },
	{ 0xAD, "Fronius IG Plus 100V-1 1-phase inverter" },
	{ 0xAC, "Fronius IG Plus 100V-2 2-phase inverter" },
	{ 0x84, "Fronius IG Plus 100V-3 3-phase inverter" },
	{ 0xAB, "Fronius IG Plus 120V-3 3-phase inverter" },
	{ 0xAA, "Fronius IG Plus 150V-3 3-phase inverter" },
	{ 0xA9, "Fronius IG Plus V 3.0-1 UNI 1-phase inverter" },
	{ 0xA8, "Fronius IG Plus V 3.8-1 UNI 1-phase inverter" },
	{ 0xA7, "Fronius IG Plus V 5.0-1 UNI 1-phase inverter" },
	{ 0xA6, "Fronius IG Plus V 6.0-1 UNI 1-phase inverter" },
	{ 0xA5, "Fronius IG Plus V 7.5-1 UNI 1-phase inverter" },
	{ 0xA4, "Fronius IG Plus V 10.0-1 UNI 1-phase inverter" },
	{ 0x87, "Fronius IG Plus V 10.0-3 Delta 3-phase inverter" },
	{ 0xA3, "Fronius IG Plus V 11.4-1 UNI 1-phase inverter" },
	{ 0xA2, "Fronius IG Plus V 11.4-3 DELTA 3-phase inverter" },
	{ 0xA1, "Fronius IG Plus V 12.0-3 WYE 3-phase inverter" },
	{ 0xA0, "Fronius IG Plus 50V-1 Dummy inverter" },
	{ 0x9F, "Fronius IG Plus 100V-2 Dummy inverter" },
	{ 0x9E, "Fronius IG Plus 150V-3 Dummy inverter" },
	{ 0x9D, "Fronius IG Plus V 3.8-1 Dummy inverter" },
	{ 0x9C, "Fronius IG Plus V 7.5-1 Dummy inverter" },
	{ 0x9B, "Fronius IG Plus V 12.0-3 Dummy inverter" },
	{ 0xBC, "Fronius CL 36.0 3-phase inverter" },
	{ 0xBD, "Fronius CL 48.0 3-phase inverter" },
	{ 0xC9, "Fronius CL 60.0 3-phase inverter" },
	{ 0xB9, "Fronius CL 36.0 WYE277 3-phase inverter" },
	{ 0xBA, "Fronius CL 48.0 WYE277 3-phase inverter" },
	{ 0xBB, "Fronius CL 60.0 WYE277 3-phase inverter" },
	{ 0xB6, "Fronius CL 33.3 Delta 3-phase inverter" },
	{ 0xB7, "Fronius CL 44.4 Delta 3-phase inverter" },
	{ 0xB8, "Fronius CL 55.5 Delta 3-phase inverter" },
	{ 0x9A, "Fronius CL 60.0 Dummy inverter" },
	{ 0x99, "Fronius CL 55.5 Delta Dummy inverter" },
	{ 0x98, "Fronius CL 60.0 WYE277 Dummy inverter" },
};

const char* getInverterModel(unsigned char id)
{
	unsigned int	i;

	if (Verbose != 0)
	{
		printf("searching for inverter type %x\n", id);
	}

	for (i = 0; i < NUM_ELEMENTS(Inverters); i++)
	{
		if (Inverters[i].id == id)
		{
			return Inverters[i].model;
		}
	}

	return "invalid inverter type";
}
